package cn.com.qmth.examcloud.core.basic.service.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.poi.ExcelReader;
import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.SpecialtyEntity;
import cn.com.qmth.examcloud.core.basic.service.SpecialtyService;
import cn.com.qmth.examcloud.core.basic.service.bean.SpecialtyInfo;

@Service
public class SpecialtyServiceImpl implements SpecialtyService {

	@Autowired
	SpecialtyRepo specialtyRepo;

	private static final String[] EXCEL_HEADER = new String[]{"专业名称", "专业代码"};

	@Override
	public SpecialtyEntity saveSpecialty(SpecialtyInfo info) {
		Long id = info.getId();
		String code = info.getCode();
		Boolean enable = info.getEnable();
		Long rootOrgId = info.getRootOrgId();
		String name = info.getName();

		if (null == rootOrgId) {
			throw new StatusException("B-620001", "rootOrgId is null");
		}
		SpecialtyEntity entity = null;

		if (null != id) {
			entity = specialtyRepo.findOne(id);
			if (null == entity) {
				throw new StatusException("B-620001", "id is wrong");
			}
			if (!entity.getRootOrgId().equals(rootOrgId)) {
				throw new StatusException("B-620001", "rootOrgId or id is worng");
			}
		} else if (StringUtils.isNotBlank(code)) {
			entity = specialtyRepo.findByRootOrgIdAndCode(rootOrgId, code);
			if (null == entity) {
				if (StringUtils.isBlank(name)) {
					throw new StatusException("B-620001", "name is blank");
				}
				entity = new SpecialtyEntity();
				entity.setCode(code);
				entity.setName(name);
				entity.setRootOrgId(rootOrgId);
				entity.setEnable(true);
			}
		} else {
			throw new StatusException("B-620001", "id and code can not be all null");
		}

		if (null != enable) {
			entity.setEnable(enable);
		}
		if (StringUtils.isNotBlank(name)) {
			entity.setName(name);
		}

		SpecialtyEntity saved = specialtyRepo.save(entity);

		return saved;
	}

	@Override
	public List<Map<String, Object>> importSpecialty(Long rootOrgId, File file) {
		List<String[]> lineList = null;
		try {
			lineList = ExcelReader.readSheetBySax(PathUtil.getCanonicalPath(file), 1, 2);
		} catch (Exception e) {
			throw new StatusException("B-100110", "Excel 解析失败");
		}

		if (CollectionUtils.isEmpty(lineList)) {
			throw new StatusException("B-100111", "Excel无内容");
		}

		if (10001 < lineList.size()) {
			throw new StatusException("B-100112", "数据行数不能超过10000");
		}

		List<Map<String, Object>> failRecords = Collections
				.synchronizedList(new ArrayList<Map<String, Object>>());

		List<SpecialtyEntity> courseList = Lists.newArrayList();

		for (int i = 0; i < lineList.size(); i++) {
			String[] line = lineList.get(i);
			if (0 == i) {
				if (headerError(line)) {
					throw new StatusException("B-100111", "Excel表头错误");
				}
				continue;
			}

			boolean hasError = false;
			StringBuilder msg = new StringBuilder();

			SpecialtyEntity specialty = new SpecialtyEntity();
			specialty.setRootOrgId(rootOrgId);
			specialty.setEnable(true);

			String name = trimAndNullIfBlank(line[0]);
			if (StringUtils.isBlank(name)) {
				msg.append("  课程名称不能为空");
				hasError = true;
			} else if (name.length() > 30) {
				msg.append("  课程名称不能超过30个字符");
				hasError = true;
			}
			specialty.setName(name);

			String code = trimAndNullIfBlank(line[1]);
			if (StringUtils.isBlank(code)) {
				msg.append("  课程代码不能为空");
				hasError = true;
			} else if (code.length() > 30) {
				msg.append("  课程代码不能超过30个字符");
				hasError = true;
			}

			specialty.setCode(code);

			if (hasError) {
				failRecords.add(newError(i + 1, msg.toString()));
			} else {
				courseList.add(specialty);
			}

		}

		if (CollectionUtils.isNotEmpty(failRecords)) {
			return failRecords;
		}

		for (SpecialtyEntity cur : courseList) {
			SpecialtyEntity query = specialtyRepo.findByRootOrgIdAndCode(cur.getRootOrgId(),
					cur.getCode());

			if (null != query) {
				query.setName(cur.getName());
				specialtyRepo.save(query);
			} else {
				specialtyRepo.save(cur);
			}
		}

		return failRecords;
	}

	private Map<String, Object> newError(int lineNum, String msg) {
		Map<String, Object> map = Maps.newHashMap();
		map.put("lineNum", lineNum);
		map.put("msg", msg);
		return map;
	}

	private String trimAndNullIfBlank(String s) {
		if (StringUtils.isBlank(s)) {
			return null;
		}
		return s.trim();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param header
	 */
	private boolean headerError(String[] header) {
		for (int i = 0; i < EXCEL_HEADER.length; i++) {
			if (!EXCEL_HEADER[i].equals(header[i].trim())) {
				return true;
			}
		}
		return false;
	}

}
